#include "stdafx.h"
#include "registry.h"
#include "regvars.h"

/****************************************************************************
*                             RegistryVar::getInt
* Inputs:
*       UINT id: ID value
*       DWORD & value: Place to put value
*       DWORD def: Default value (= 0)
*       HKEY root: default HKEY_CURRENT_USER
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Retrieves the integer value
****************************************************************************/

BOOL RegistryVar::getInt(UINT id, DWORD & value, DWORD def, HKEY root)
    {
     DWORD v;
     CString pathname;
     pathname.LoadString(id);
     if(!GetRegistryInt(root, pathname, v))
        { /* use default */
         value = def;
         if(::GetLastError() == ERROR_FILE_NOT_FOUND)
            return TRUE;
         return FALSE;
        } /* use default */
     value = v;
     return TRUE;
    }


/****************************************************************************
*                             RegistryVar::setInt
* Inputs:
*       UINT id: ID to set
*       DWORD value: Value to set
*       HKEY root: default HKEY_CURRENT_USER
* Result: BOOL
*       
* Effect: 
*       Sets the registry key
****************************************************************************/

BOOL  RegistryVar::setInt(UINT id, DWORD value, HKEY root)
    {
     CString pathname;

     pathname.LoadString(id);
     return SetRegistryInt(root, pathname, value);
    }

/****************************************************************************
*                           RegistryVar::setString
* Inputs:
*       UINT id: ID of variable in 
*       LPCTSTR value: String value to save
*       HKEY root: default HKEY_CURRENT_USER
* Result: void
*       
* Effect: 
*       Stores the string value in the registry
****************************************************************************/

BOOL RegistryVar::setString(UINT id, LPCTSTR value, HKEY root)
    {
     CString pathname;

     pathname.LoadString(id);

     return SetRegistryString(root, pathname, value);
    }

/****************************************************************************
*                           RegistryVar::getString
* Inputs:
*       UINT id: ID of subkey
*       CString & s: Place to put string
*       LPCTSTR def: Default value, or NULL
*       HKEY root: default HKEY_CURRENT_USER
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Loads the string from the registry
****************************************************************************/

BOOL RegistryVar::getString(UINT id, CString & value, LPCTSTR def, HKEY root)
    {
     CString pathname;

     pathname.LoadString(id);

     if(!GetRegistryString(root, pathname, value))
        { /* failed */
         if(def != NULL)
            value = def;
         else
            value = _T("");
         return ::GetLastError() == ERROR_FILE_NOT_FOUND;
        } /* failed */
     return TRUE;
    }

/****************************************************************************
*                            RegistryVar::getGUID
* Inputs:
*       UINT id: ID of variable to load
*       GUID & value: GUID
*       GUID * def: Default, or NULL if no default
*       HKEY root: Root of key
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Loads a GUID from the Registry
****************************************************************************/

BOOL RegistryVar::getGUID(UINT id, GUID & value, const GUID * def, HKEY root)
    {
     CString pathname;

     pathname.LoadString(id);
     if(!GetRegistryGUID(root, pathname, value))
        { /* failed */
         if(def != NULL)
            value = *def;
         else
            ::ZeroMemory(&value, sizeof(GUID));
         return ::GetLastError() == ERROR_ALREADY_EXISTS;
        } /* failed */
     return TRUE;
    } // RegistryVar::getGUID

/****************************************************************************
*                            RegistryVar::setGUID
* Inputs:
*       UINT id: String ID of var
*       const GUID &value: GUID to store
*       HKEY root: Root of key (default HKEY_CURRENT_USER)
* Result: BOOL
*       
* Effect: 
*       Stores the GUID
****************************************************************************/

BOOL RegistryVar::setGUID(UINT id, const GUID &value, HKEY root)
    {
     CString pathname;
     pathname.LoadString(id);
     return SetRegistryGUID(root, pathname, value);
    } // RegistryVar::setGUID

/****************************************************************************
*                              RegistryInt::load
* Result: BOOL
*       TRUE if loaded successfully
*       FALSE if no load occurred
* Effect: 
*       Loads the 'value'
****************************************************************************/

BOOL RegistryInt::load(DWORD def)
    {
     return getInt(id, value, def, root);
    }

/****************************************************************************
*                             RegistryInt::store
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Stores the 'value'
****************************************************************************/

BOOL RegistryInt::store()
    {
     return setInt(id, value, root);
    }

/****************************************************************************
*                            RegistryString::load
* Inputs:
*       LPCTSTR def: Default value, or NULL if no default
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Loads the string
****************************************************************************/
 
BOOL RegistryString::load(LPCTSTR def)
    {
     return getString(id, value, def, root);
    }

/****************************************************************************
*                            RegistryString::store
* Result: void
*       
* Effect: 
*       Stores the variable
****************************************************************************/

BOOL RegistryString::store()
    {
     return setString(id, (LPCTSTR)value, root);
    }

/****************************************************************************
*                             RegistryGUID::load
* Inputs:
*       GUID * def: Default value
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Loads the GUID
****************************************************************************/

BOOL RegistryGUID::load(const GUID * def)
    {
     return getGUID(id, value, def, root);
    } // RegistryGUID::load

/****************************************************************************
*                             RegistryGUID::store
* Result: void
*       
* Effect: 
*       Stores the GUID in the Registry
****************************************************************************/

BOOL RegistryGUID::store()
    {
     return setGUID(id, value, root);
    } // RegistryGUID::store
