#include "stdafx.h"
#include "resource.h"
#include "ErrorString.h"
#include "Filter.h"

/****************************************************************************
*                                Filter::Find
* Inputs:
*       const CString & s: String to find
* Result: int
*       Index of Filter entry, -1 if not found
****************************************************************************/

int Filter::Find(const CString & s)
    {
     for(int i = 0; i < table.GetSize(); i++)
        { /* locate item */
         if(table[i].GetText() == s)
            return i;
        } /* locate item */
     return -1; // not found
    } // Filter::Find

/****************************************************************************
*                                 Filter::Add
* Inputs:
*       const CString & s: String to add
*       BOOL enable (=TRUE): enable state of filter (ignored if filter already exists)
* Result: int
*       Index of element in array (index of existing filter if it already exists)
* Effect: 
*       Adds the element if it does not already exist
****************************************************************************/

int Filter::Add(const CString & s, BOOL enable /* = TRUE */)
    {
     int i = Find(s);
     if(i >= 0)
        { /* already exists */
         return i;
        } /* already exists */

     i = table.Add(FilterInfo(s, enable));
     return i;
    } // Filter::Add

/****************************************************************************
*                             Filter::IsFiltered
* Inputs:
*       const CString & s: String to match
* Result: BOOL
*       TRUE if it matches an enabled filter
*       FALSE if it matches a disabled filter, or does not match
****************************************************************************/

BOOL Filter::IsFiltered(const CString & s)
    {
     int i = Find(s);
     if(i < 0)
        return FALSE;
     return table[i].GetEnabled();
    } // Filter::IsFiltered

/****************************************************************************
*                                Filter::Save
* Inputs:
*       CString fname: Filename, or empty string to prompt
* Result: BOOL
*       TRUE if successful (fname is updated if it was empty)
*       FALSE if failure
* Effect: 
*       Saves the filter information
****************************************************************************/

BOOL  Filter::Save(CString & fname)
    {
     CString filename = fname;

     if(filename.IsEmpty())
        { /* prompt for name */
         CString filter;
         filter.LoadString(IDS_FILTER_FILTER);

         CFileDialog dlg(FALSE, _T(".flt"), _T("Filters.flt"),
                         OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, filter);

         if(dlg.DoModal() != IDOK)
            return FALSE;

         filename = dlg.GetPathName();
        } /* prompt for name */

     CStdioFile f;
     if(!f.Open(filename, CFile::modeWrite | CFile::modeCreate))
        { /* failed */
         DWORD err = ::GetLastError();
         CString fmt;
         fmt.LoadString(IDS_OPEN_FAILURE);
         CString msg;
         msg.Format(fmt, filename, ErrorString(err));
         AfxMessageBox(msg, MB_ICONERROR | MB_OK);
         return FALSE;
        } /* failed */

     for(int i = 0; i < table.GetSize(); i++)
        { /* write each line */
         CString s;
         s.Format(_T("%01d,%s\n"), table[i].GetEnabled() ? 1 : 0, table[i].GetText());
         f.WriteString(s);
        } /* write each line */
     f.Close();
     fname = filename;
     return TRUE;
    } // Filter::Save

/****************************************************************************
*                                Filter::Load
* Inputs:
*       const CString & fname: Filename, or empty string to prompt
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Loads the elements in. Note that these REPLACE any existing elements
*       if the file is opened at all
****************************************************************************/

BOOL Filter::Load(const CString & fname)
    {
     CString filename = fname;

     if(filename.IsEmpty())
        { /* prompt for name */
         CString filter;
         filter.LoadString(IDS_FILTER_FILTER);

         CFileDialog dlg(TRUE, _T(".flt"), _T("Filters.flt"),
                         OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, filter);

         if(dlg.DoModal() != IDOK)
            return FALSE;

         filename = dlg.GetPathName();
        } /* prompt for name */

     CStdioFile f;
     if(!f.Open(filename, CFile::modeRead))
        { /* failed */
         DWORD err = ::GetLastError();
         CString fmt;
         fmt.LoadString(IDS_OPEN_FAILURE);
         CString msg;
         msg.Format(fmt, filename, ErrorString(err));
         AfxMessageBox(msg, MB_ICONERROR | MB_OK);
         return FALSE;
        } /* failed */


     BOOL cleared = FALSE;

     CString line;
     TRY {
     while(f.ReadString(line))
        { /* got line */
         if( (line[0] == _T('0') || line[0] == _T('1')) && line[1] == _T(','))
            { /* may be valid */
             if(!cleared)
                table.RemoveAll();
             cleared = TRUE;
             CString text = line.Mid(2);
             text.TrimLeft();
             Add(text, atoi(line));
            } /* may be valid */
         else
            { /* line is erroneous */
             // Line has bad format. Continue reading more lines? [Yes][No]
             CString fmt;
             fmt.LoadString(IDS_BAD_FILTER_LINE);
             CString msg;
             msg.Format(fmt, line);
             switch(AfxMessageBox(msg, MB_ICONERROR | MB_YESNO))
                { /* message box */
                 case IDYES:
                    break;
                 case IDNO:
                    f.Close();
                    return FALSE;
                } /* message box */
            } /* line is erroneous */
        } /* got line */
     } 
     CATCH(CFileException, e)
     {
         e->Delete();
     }
     END_CATCH

     f.Close();
     return TRUE;
    } // Filter::Load

/****************************************************************************
*                              Filter::SetModify
* Inputs:
*       BOOL mode = TRUE: Modified mode. 
*                       If FALSE, recomputes the checksum.
* Result: void
*       
* Effect: 
*       If mode == TRUE, forces the GetModify result to be TRUE, by forcing
*               the checksum to 0.
****************************************************************************/

void Filter::SetModify(BOOL mode)
    {
     if(mode)
        { /* force modify detection */
         Checksum sum;
         sum.Add((int)1);
         chksum = sum;
        } /* force modify detection */
     else
        { /* force unmodified */
         chksum = computeChecksum();
        } /* force unmodified */
    } // Filter::SetModify

/****************************************************************************
*                              Filter::GetModify
* Result: BOOL
*       TRUE if modified
****************************************************************************/

BOOL Filter::GetModify()
    {
     Checksum sum;
     sum = computeChecksum();
     return (sum != chksum);
    } // Filter::GetModify

/****************************************************************************
*                           Filter::computeChecksum
* Result: Checksum
*       
* Notes: 
*       Computes the checksum based on the contents and checked state
****************************************************************************/

Checksum Filter::computeChecksum()
    {
     Checksum sum;
     for(int i = 0; i < table.GetSize(); i++)
        { /* iterate */
         sum.Add(table[i].GetText());
         sum.Add(table[i].GetEnabled());
        } /* iterate */
     return sum;
    } // Filter::computeChecksum
